/*
 * Copyright 2009-2010 The MegaDrive.Org Team
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 * 
 *    1. Redistributions of source code must retain the above copyright notice, this list of
 *       conditions and the following disclaimer.
 * 
 *    2. Redistributions in binary form must reproduce the above copyright notice, this list
 *       of conditions and the following disclaimer in the documentation and/or other materials
 *       provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE MEGADRIVE.ORG TEAM ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EMMANUEL VADOT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* $Id: vdp.h,v 1.5 2010/01/11 06:32:36 elbarto Exp $ */

/*! \file vdp.h
    \brief VDP related function

*/

#include <types.h>

/*! \def REG_NULL
    \brief Use to set a vdp register to 0
 */

#define	REG_NULL	0


/*! \def REG0_HVCOUNTER
    \brief Stop HV counter
 */
/*! \def REG0_BASE
    \brief Base for REG0
 */
/*! \def REG0_HINT
    \brief Enable HINT
 */

#define	REG0_HVCOUNTER	0x2
#define REG0_BASE	0x4
#define	REG0_HINT	0x10

/*! \def REG1_BASE
    \brief Base for REG1
 */
/*! \def REG1_PAL
    \brief Enable PAL mode
 */
/*! \def REG1_DMA
    \brief Enable DMA
 */
/*! \def REG1_VINT
    \brief Enable VINT
 */
/*! \def REG1_DISPLAY
    \brief Enable DISPLAY
 */

#define	REG1_BASE	0x4
#define	REG1_PAL	0x8
#define	REG1_DMA	0x10
#define	REG1_VINT	0x20
#define	REG1_DISPLAY	0x40

/*! \def REG2_PLANEA(addr)
    \brief Set PLANEA address
 */

#define	REG2_PLANEA(addr)	((addr & 0xE000) >> 10)

/*! \def REG3_WINDOW(addr)
    \brief Set WINDOW address
 */

#define REG3_WINDOW(addr)	((addr & 0xF800) >> 10)

/*! \def REG4_PLANEB(addr)
    \brief Set PLANEB address
 */

#define REG4_PLANEB(addr)	((addr & 0xE000) >> 13)

/*! \def REG5_SPRITE(addr)
    \brief Set SPRITE address
 */

#define REG5_SPRITE(addr)	((addr & 0xFE00) >> 9)

/*! \def REG7_PAL(pal)
    \brief Select the palette pal for the bg color
 */
/*! \def REG7_COLOR(color)
    \brief Select the color color for the bg color
 */

#define REG7_PAL(pal)	((pal & 0x3) << 4)
#define	REG7_COLOR(color)	(color & 0xF)

/*! \def REG11_HFULL
    \brief Set horizontal full scroll mode
 */
/*! \def REG11_H1CELL
    \brief Set horizontal 1 cell scroll mode
 */
/*! \def REG11_H1LINE
    \brief Set horizontal 1 line scroll mode
 */
/*! \def REG11_VFULL
    \brief Set vertical full scroll mode
 */
/*! \def REG11_V2CELL
    \brief Set vertical 2 cell scroll mode
 */
/*! \def REG11_EINT
    \brief Enable external interupt
 */

#define	REG11_HFULL	0x0
#define	REG11_H1CELL	0x2
#define	REG11_H1LINE	0x3
#define	REG11_VFULL	0x0
#define	REG11_V2CELL	0x4
#define	REG11_EINT	0x8

/*! \def REG12_40CELL
    \brief Set 40 cell mode
 */
/*! \def REG12_SHADOW
    \brief Enable Shadow/Highlight
 */
/*! \def REG12_INTERLACE
    \brief Select interlace mode
 */
/*! \def REG12_DINTERLACE
    \brief Select double interlace mode
 */

#define	REG12_40CELL		0x81
#define	REG12_SHADOW		0x8
#define	REG12_INTERLACE		0x2
#define	REG12_DINTERLACE	0x6

/*! \def REG13_HSCROLL(addr)
    \brief Set HSCROLL address
 */

#define	REG13_HSCROLL(addr)	((addr & 0xFC00) >> 10)

/*! \def REG15_AUTOINC(x)
    \brief Set VDP autoinc to x
 */

#define	REG15_AUTOINC(x)	x

/*! \def REG16_32X32
    \brief Set scroll size to 32x32
 */
/*! \def REG16_32X64
    \brief Set scroll size to 32x64
 */
/*! \def REG16_32X128
    \brief Set scroll size to 32x128
 */
/*! \def REG16_64X32
    \brief Set scroll size to 64x32
 */
/*! \def REG16_64X64
    \brief Set scroll size to 64x64
 */
/*! \def REG16_64X128
    \brief 64x128
 */
/*! \def REG16_128X32
    \brief Set scroll size to 128x32
 */
/*! \def REG16_128X64
    \brief Set scroll size to 128x64
 */
/*! \def REG16_128X128
    \brief Set scroll size to 128x128
 */

#define	REG16_32X32	0x0
#define	REG16_32X64	0x10
#define	REG16_32X128	0x30
#define	REG16_64X32	0x1
#define	REG16_64X64	0x11
#define	REG16_64X128	0x31
#define	REG16_128X32	0x3
#define	REG16_128X64	0x13
#define	REG16_128X128	0x33

/*! \def SPRITE_SIZE_8X8
    \brief Set sprite size to 8x8
 */
/*! \def SPRITE_SIZE_8X16
    \brief Set sprite size to 8x16
 */
/*! \def SPRITE_SIZE_8X24
    \brief Set sprite size to 8x24
 */
/*! \def SPRITE_SIZE_8X32
    \brief Set sprite size to 8x32
 */
/*! \def SPRITE_SIZE_16X8
    \brief Set sprite size to 16x8
 */
/*! \def SPRITE_SIZE_16X16
    \brief Set sprite size to 16x16
 */
/*! \def SPRITE_SIZE_16X24
    \brief Set sprite size to 16x24
 */
/*! \def SPRITE_SIZE_16X32
    \brief Set sprite size to 16x32
 */
/*! \def SPRITE_SIZE_24X8
    \brief Set sprite size to 24x8
 */
/*! \def SPRITE_SIZE_24X16
    \brief Set sprite size to 24x16
 */
/*! \def SPRITE_SIZE_24X24
    \brief Set sprite size to 24x24
 */
/*! \def SPRITE_SIZE_24X32
    \brief Set sprite size to 24x32
 */
/*! \def SPRITE_SIZE_32X8
    \brief Set sprite size to 32x8
 */
/*! \def SPRITE_SIZE_32X16
    \brief Set sprite size to 32x16
 */
/*! \def SPRITE_SIZE_32X24
    \brief Set sprite size to 32x24
 */
/*! \def SPRITE_SIZE_32X32
    \brief Set sprite size to 32x32
 */

#define	SPRITE_SIZE_8X8		(0x0 << 8)
#define	SPRITE_SIZE_8X16	(0x1 << 8)
#define	SPRITE_SIZE_8X24	(0x2 << 8)
#define	SPRITE_SIZE_8X32	(0x3 << 8)
#define	SPRITE_SIZE_16X8	(0x4 << 8)
#define	SPRITE_SIZE_16X16	(0x5 << 8)
#define	SPRITE_SIZE_16X24	(0x6 << 8)
#define	SPRITE_SIZE_16X32	(0x7 << 8)
#define	SPRITE_SIZE_24X8	(0x8 << 8)
#define	SPRITE_SIZE_24X16	(0x9 << 8)
#define	SPRITE_SIZE_24X24	(0xA << 8)
#define	SPRITE_SIZE_24X32	(0xB << 8)
#define	SPRITE_SIZE_32X8	(0xC << 8)
#define	SPRITE_SIZE_32X16	(0xD << 8)
#define	SPRITE_SIZE_32X24	(0xE << 8)
#define	SPRITE_SIZE_32X32	(0xF << 8)

/*! \def CRAM_WRITE
    \brief Set write access to CRAM
 */
/*! \def CRAM_READ
    \brief Set read access to CRAM
 */
/*! \def VRAM_WRITE
    \brief Set write access to VRAM
 */
/*! \def VRAM_READ
    \brief Set read access to VRAM
 */
/*! \def VSRAM_WRITE
    \brief Set write access to VSRAM
 */
/*! \def VSRAM_READ
    \brief Set read access to VSRAM
 */

#define	CRAM_WRITE	0x3
#define	CRAM_READ	0x8
#define	VRAM_WRITE	0x1
#define	VRAM_READ	0x0
#define	VSRAM_WRITE	0x5
#define	VSRAM_READ	0x4

/*! \def DMA_TO_VRAM
    \brief Set destination to VRAM for DMA
*/
/*! \def DMA_TO_CRAM
    \brief Set destination to CRAM for DMA
*/
/*! \def DMA_TO_VSRAM
    \brief Set destination to VSRAM for DMA
*/

#define	DMA_TO_VRAM	0x1
#define	DMA_TO_CRAM	0x3
#define	DMA_TO_VSRAM	0x5

/*! \def VDP_BUILD_TILE(n, vflip, hflip, pal, shadow)
    \brief Build a tile definition

    Build a tile definition with
    n = tilenumber
    vflip = flip the tile verticaly
    hflip = flip the tile horizontaly
    pal = palette to use
    shadow = enable shadow
 */

#define	VDP_BUILD_TILE(n, vflip, hflip, pal, shadow)	\
  (n | (vflip << 11) | (hflip << 12) | (pal << 13) | (shadow << 15))

/*! \fn void vdp_pointto(u32 addr, u8 access)
    \brief Set the VDP address to addr in the correct region and access mode
    \param addr the address to point to
    \param access the region and access type
 */

void	vdp_pointto(u32 addr, u8 access);

/*! \fn void vdp_setreg(u32 reg, u32 data)
    \brief Set a VDP register
    \param reg The register number
    \param data The register data
 */

void	vdp_setreg(u32 reg, u32 data);

/*! \fn void vdp_write(u16 data)
    \brief Write 16bits to the VDP
    \param data Data to write
*/

void	vdp_write(u16 data);

/*! \fn void vdp_write16(u16 data)
    \brief Write 16bits to the VDP
    \param data Data to write
*/

void	vdp_write16(u16 data);

/*! \fn void vdp_write32(u32 data)
    \brief Write 16bits to the VDP
    \param data Data to write
*/

void	vdp_write32(u32 data);

/*! \fn void vdp_writetocram(u8 size, const void *src, u8 dst)
    \brief Write a palette in the VDP
    \param size The amount of data to write in byte
    \param src The address of the data to write
    \param dst The destination address in CRAM
*/

void	vdp_writetocram(u8 size, const void *src, u8 dst);

/*! \fn void vdp_writetovram(u16 size, const void *from, u16 dst)
    \brief Write size byte of data from *src
    \param size The amount of data to write in byte
    \param from The address of the data to write
    \param dst The destination address in VRAM
 */

void	vdp_writetovram(u16 size, const void *from, u16 dst);

/*! \fn void vdp_writetovsram(u16 size, const void *from, u16 dst)
    \brief Write size byte of data from *src
    \param size The amount of data to write in byte
    \param from The address of the data to write
    \param dst The destination address in VRAM
 */

void	vdp_writetovsram(u16 size, const void *from, u16 dst);

/*! \fn void vdp_dma(u16 size, const void *from, u16 dst, u8 region)
    \brief Write size byte of data from *src to dst using the DMA
    \param size The amount of data to write in byte
    \param from The address of the data to write
    \param dst The destination address in VDP
    \param region The destination RAM in VDP (VRAM, CRAM or VSRAM)
 */

void	vdp_dma(u16 size, const void *from, u16 dst, u8 region);

/*! \fn void vdp_dmafill(u16 size, u16 dst, u16 data)
    \brief Fill size byte with data to dst using the DMA
    \param size The amount of data to write in byte
    \param dst The destination address in VDP
    \param data The data to fill
 */

void	vdp_dmafill(u16 size, u16 dst, u16 data);

/*! \fn void vdp_waitdma(void)
    \brief Wait for dma completion
*/

void	vdp_waitdma(void);

/*! \fn void vdp_clrvram(void)
    \brief Clear the VDP VRAM

    You have to set the autoinc register (#15) to 2 before calling this function
 */

void	vdp_clrvram(void);

/*! \fn void vdp_clrvsram(void)
    \brief Clear the VDP VSRAM

    You have to set the autoinc register (#15) to 2 before calling this function
 */

void	vdp_clrvsram(void);

/*! \fn void vdp_waitvsync(void)
    \brief Wait for the next VBLANK
 */

void	vdp_waitvsync(void);
